
#include "inc.h"

#include "minix.h"
#include "iwram_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "gbfs.h"
#include "lib.h"
#include "sizuku.h"

#include "screen.h"

//---------------------------------------------------------------------------
//lib.c
extern ST_TIMER Timer;

//---------------------------------------------------------------------------
ST_FONT_LEAF   FontLeaf;
ST_SCREEN_IMG  ScreenImg  EWRAM_DATA;
ST_SCREEN_FONT ScreenFont;
ST_CURSOR      Cursor;

u32 dummy ALIGN(4);
//---------------------------------------------------------------------------
EWRAM_CODE void FontLeafInit()
{
	ST_FONT_LEAF* p = &FontLeaf;

	p->pAttr[0] = (u8*)&k12x10wBitmap;
	p->pAttr[1] = (u8*)&k12x10gBitmap;
	p->attrNo   = 0;
	p->pDat     = p->pAttr[0];
	p->imgCx    = K12X10_IMG_CX;
	p->cx       = K12X10_FONT_CX;
	p->cy       = K12X10_FONT_CY;
	p->pTblSJIS = (u16*)&sjis2leaf_txt;
	p->tblCnt   = SJIS2LEAF_CODE_CNT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void FontLeafDrawChr(u16 x, u16 y, u16 code)
{
	u8 flag = 0x00;

	if(x & 0x01) flag |= 0x01;
	if(y & 0x01) flag |= 0x10;

	//x, yW̋Aɂďݕ@ύX܂
	switch(flag)
	{
		case 0x00: FontLeafDrawChrType1(x, y, code); break;
		case 0x01: FontLeafDrawChrType2(x, y, code); break;
		case 0x10: FontLeafDrawChrType3(x, y, code); break;
		case 0x11: FontLeafDrawChrType4(x, y, code); break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void FontLeafDrawChrType1(u16 x, u16 y, u16 code)
{
	ST_FONT_LEAF* p = &FontLeaf;

	u16* pScr1 = OAMdata + (y * 768) + (x * 24);
	u16* pScr2 = pScr1 + 16;
	u16* pScr3 = pScr1 + 512;
	u16* pScr4 = pScr1 + 512 + 16;
	u16* pDat  = (u16*)p->pDat + code * 3;

	u16 i;
	for(i=0; i<8; i++)
	{
		*pScr1++ = *(pDat + 0);
		*pScr1++ = *(pDat + 1);
		*pScr2++ = *(pDat + 2);
		pScr2++;

		pDat += (p->imgCx / 4);
	}

	for(i=0; i<2; i++)
	{
		*pScr3++ = *(pDat + 0);
		*pScr3++ = *(pDat + 1);
		*pScr4++ = *(pDat + 2);
		pScr4++;

		pDat += (p->imgCx / 4);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void FontLeafDrawChrType2(u16 x, u16 y, u16 code)
{
	ST_FONT_LEAF* p = &FontLeaf;

	u16* pScr1 = OAMdata + (16+1) + (y * 768) + ((x-1) * 24);
	u16* pScr2 = pScr1 + 16 - 1;
	u16* pScr3 = pScr1 + 512;
	u16* pScr4 = pScr2 + 512;
	u16* pDat  = (u16*)p->pDat + code * 3;

	u16 i;
	for(i=0; i<8; i++)
	{
		*pScr1++ = *(pDat + 0);
		pScr1++;

		*pScr2++ = *(pDat + 1);
		*pScr2++ = *(pDat + 2);

		pDat += (p->imgCx / 4);
	}

	for(i=0; i<2; i++)
	{
		*pScr3++ = *(pDat + 0);
		pScr3++;

		*pScr4++ = *(pDat + 1);
		*pScr4++ = *(pDat + 2);

		pDat += (p->imgCx / 4);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void FontLeafDrawChrType3(u16 x, u16 y, u16 code)
{
	ST_FONT_LEAF* p = &FontLeaf;

	u16* pScr1 = OAMdata + (512 + 8) + ((y-1) * 768) + (x * 24);
	u16* pScr2 = pScr1 + 16;
	u16* pScr3 = pScr1 + 512 - 8;
	u16* pScr4 = pScr2 + 512 - 8;
	u16* pDat  = (u16*)p->pDat + code * 3;

	u16 i;
	for(i=0; i<4; i++)
	{
		*pScr1++ = *(pDat + 0);
		*pScr1++ = *(pDat + 1);
		*pScr2++ = *(pDat + 2);
		pScr2++;

		pDat += (p->imgCx / 4);
	}

	for(i=0; i<6; i++)
	{
		*pScr3++ = *(pDat + 0);
		*pScr3++ = *(pDat + 1);
		*pScr4++ = *(pDat + 2);
		pScr4++;

		pDat += (p->imgCx / 4);
	}

}
//---------------------------------------------------------------------------
EWRAM_CODE void FontLeafDrawChrType4(u16 x, u16 y, u16 code)
{
	ST_FONT_LEAF* p = &FontLeaf;

	u16* pScr1 = OAMdata + (528 + 8 + 1) + ((y-1) * 768) + ((x-1) * 24);
	u16* pScr2 = pScr1 +  16 - 1;
	u16* pScr3 = pScr1 + 512 - 8;
	u16* pScr4 = pScr3 +  16 - 1;
	u16* pDat  = (u16*)p->pDat + code * 3;

	u16 i;
	for(i=0; i<4; i++)
	{
		*pScr1++ = *(pDat + 0);
		pScr1++;

		*pScr2++ = *(pDat + 1);
		*pScr2++ = *(pDat + 2);

		pDat += (p->imgCx / 4);
	}

	for(i=0; i<6; i++)
	{
		*pScr3++ = *(pDat + 0);
		pScr3++;

		*pScr4++ = *(pDat + 1);
		*pScr4++ = *(pDat + 2);

		pDat += (p->imgCx / 4);
	}
}
//---------------------------------------------------------------------------
//SJIS -> Leaf
EWRAM_CODE u16 FontLeafGetLeafCode(u16 code)
{
	ST_FONT_LEAF* p = &FontLeaf;
	u16 i;

	for(i=0; i<p->tblCnt; i++)
	{
		if(code == p->pTblSJIS[i])
		{
			return i;
		}
	}

	TRACEOUT("Err: FontLeafGetLeafCode\n");
	return 0;
}
//---------------------------------------------------------------------------
//Leaf -> SJIS
EWRAM_CODE u16 FontLeafGetSJISCode(u16 code)
{
	return FontLeaf.pTblSJIS[code];
}
//---------------------------------------------------------------------------
EWRAM_CODE void FontLeafSetAttr(u16 no)
{
	FontLeaf.pDat   = FontLeaf.pAttr[no];
	FontLeaf.attrNo = no;
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 FontLeafGetAttrNo()
{
	return FontLeaf.attrNo;
}
//---------------------------------------------------------------------------
EWRAM_CODE void FontLeafDebugCode(u16 code)
{
	u16 sjis = FontLeafGetSJISCode(code);
	char buf[16];

	buf[0] = (sjis & 0x00ff);
	buf[1] = (sjis & 0xff00) >> 8;
	buf[2] = '\0';

	TRACEOUT("%s", buf);
}





//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontInit()
{
	static const u16 x[8] = {  18,  18,  82,  82, 146, 146, 210, 210 };
	static const u16 y[8] = {  22,  86,  22,  86,  22,  86,  22,  86 };
	static const u16 s[8] = { 512, 768, 520, 776, 528, 784, 536, 792 };
	static const u16 t[8] = { SP_SQUARE, SP_SQUARE, SP_SQUARE, SP_SQUARE, SP_SQUARE, SP_SQUARE, SP_TALL, SP_TALL };

	ST_SCREEN_FONT* p = &ScreenFont;

	u16 i;
	for(i=0; i<SCREEN_FONT_SPRITE_CNT; i++)
	{
		p->in[i].attr0  = SP_COLOR_16 | (t[i]) | (y[i]);
		p->in[i].attr1  = SP_SIZE_64  | (x[i]);
		p->in[i].attr2  = s[i];
		p->in[i].attr3  = 0x0000;

		p->out[i].attr0 = SP_COLOR_16 | SP_SQUARE | (SCREEN_CY);
		p->out[i].attr1 = SP_SIZE_64;
		p->out[i].attr2 = s[i];
		p->out[i].attr3 = 0x0000;
	}

	ScreenFontClear();
	ScreenFontIn();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontIn()
{
	CpuSet((u32)ScreenFont.in, (u32)OAMmem + 512 * 8, (8*8 / 4) | CPUSET_32BIT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontOut()
{
	CpuSet((u32)ScreenFont.out, (u32)OAMmem + 512 * 8, (8*8 / 4) | CPUSET_32BIT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontClear()
{
	ScreenFontClearOAM();
	ScreenFontClearBuf();
	ScreenFontClearXY();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontClearNoBuf()
{
	ScreenFontClearOAM();
	ScreenFontClearXY();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontClearOAM()
{
	CpuSet((u32)&dummy, (u32)OAMdata, (15168 / 4) | CPUSET_32BIT | CPUSET_SRC_FIX);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontClearBuf()
{
	CpuSet((u32)&dummy, (u32)ScreenFont.buf.code,   ((SCREEN_FONT_CX * SCREEN_FONT_CY * 2) / 4) | CPUSET_32BIT | CPUSET_SRC_FIX);
	CpuSet((u32)&dummy, (u32)ScreenFont.buf.attrNo, ((SCREEN_FONT_CX * SCREEN_FONT_CY)     / 4) | CPUSET_32BIT | CPUSET_SRC_FIX);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontSetChrBuf(u16 x, u16 y, u16 code, u8 attrNo)
{
	ScreenFont.buf.code  [x][y] = code;
	ScreenFont.buf.attrNo[x][y] = attrNo;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontDrawChr(u16 x, u16 y, u16 code)
{
	FontLeafDrawChr(x, y, code);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontDrawStr(u16* buf, u16 cnt)
{
	ST_SCREEN_FONT* p = &ScreenFont;
	u8 attrNo = FontLeafGetAttrNo();

	u16 i;
	for(i=0; i<cnt; i++)
	{
		if(p->x >= (SCREEN_FONT_CX-1) && i<cnt)
		{
			ScreenFontSetNewLine();
		}

		ScreenFontSetChrBuf(p->x, p->y, buf[i], attrNo);
		ScreenFontDrawChr(p->x, p->y, buf[i]);
		p->x++;

		FontLeafDebugCode(buf[i]);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontDrawStrNoBuf(u16* buf, u16 cnt)
{
	ST_SCREEN_FONT* p = &ScreenFont;

	u16 i;
	for(i=0; i<cnt; i++)
	{
		if(p->x >= (SCREEN_FONT_CX-1) && i<cnt)
		{
			ScreenFontSetNewLine();
		}

		ScreenFontDrawChr(p->x, p->y, buf[i]);
		p->x++;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontDrawStrSJISNoBuf(u16 sx, u16 sy, u8* p)
{
	u16 sjis;
	u16 x = 0;

	while(*p != '\0')
	{
		sjis  = (*p++ & 0x00ff);
		sjis |= (*p++ & 0x00ff) << 8;

		//sɂ͑ΉĂȂł
		ScreenFontDrawChr(sx + x, sy, FontLeafGetLeafCode(sjis));
		x++;
	}
}
//---------------------------------------------------------------------------
//֑̏(ȂKȏĂ܂)
EWRAM_CODE u16 ScreenFontSetHyphenation(u16* src, u16* dst, u16 len)
{
	u16 x = ScreenFont.x;
	u16 i = 0;
	u16 j = 0;
	u16 code;

	//O
	if(x == (SCREEN_FONT_CX-2))
	{
		dst[i++] = 0x0000;		//0x0000 = LeafCode "@"
		x = 0;
	}
	else if(x >= (SCREEN_FONT_CX-1))
	{
		x = 0;
	}

	if(src[j+0] == 0x0000)
	{
		dst[i++] = 0x0000;
		j++;
		x++;
	}

	while(j < len)
	{
		code = src[j++];

		if(code == 0x0000)
		{
			continue;
		}
		else if( (x == 0) && (ScreenFontIsHyphenationBefore(code) == TRUE) )
		{
			//s
			dst[i+0] = dst[i-1];
			dst[i+1] = code;
			dst[i-1] = 0x0000;

			//ړAɋ֑`FbN܂
			if( ScreenFontIsHyphenationBefore(dst[i+0]) == FALSE )
			{
				i += 2;
				x  = 2;
			}
			else
			{
				dst[i+2] = dst[i+1];
				dst[i+1] = dst[i+0];
				dst[i+0] = dst[i-2];
				dst[i-2] = 0x0000;

				i += 3;
				x  = 3;
			}
		}
		else if( (x >= (SCREEN_FONT_CX-2)) && (ScreenFontIsHyphenationAfter(code) == TRUE) )
		{
			//s
			dst[i+1] = dst[i-1];
			dst[i+2] = code;
			dst[i+0] = 0x0000;
			dst[i-1] = 0x0000;

			i += 3;
			x  = 3;
		}
		else
		{
			dst[i] = code;
			i++;
			x++;

			if(x >= (SCREEN_FONT_CX-1))
			{
				x = 0;
			}
		}
	}

	return i;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ScreenFontIsHyphenationBefore(u16 code)
{
	switch(code)
	{
		case 0x0030:	//	TODO t@Cǂݍ܂悤ɂ
		case 0x0031:	//
		case 0x0032:	//
		case 0x0033:	//
		case 0x0057:	//B
		case 0x0058:	//A
		case 0x0059:	//E
		case 0x005a:	//c
		case 0x005c:	//x
		case 0x005e:	//v
		case 0x0060:	//j
		case 0x0061:	//H
		case 0x0062:	//I
		case 0x0063:	//!?
		case 0x0064:	//|
		case 0x0065:	//[
		case 0x0097:	//
		case 0x0098:	//
		case 0x0099:	//
		case 0x009a:	//b
		case 0x04e9:	//
		case 0x04ea:	//
		case 0x04eb:	//
		case 0x04ec:	//
		case 0x04ed:	//
		case 0x04ee:	//
		case 0x04ef:	//@
		case 0x04f0:	//B
		case 0x04f1:	//D
		case 0x04f2:	//F
		case 0x04f3:	//H
		case 0x0683:	//D
		case 0x06ff:	//`
			return TRUE;
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ScreenFontIsHyphenationAfter(u16 code)
{
	switch(code)
	{
		case 0x005b:	//w
		case 0x005d:	//u
		case 0x005f:	//i
			return TRUE;
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ScreenFontIsFull()
{
	return ScreenFont.y >= SCREEN_FONT_CY ? TRUE : FALSE;
}
//---------------------------------------------------------------------------
//ʂɕ\镶񂪁A܂邩`FbN܂
EWRAM_CODE bool ScreenFontIsOver(u16 writeCnt)
{
	u16 cnt  = (ScreenFont.x + ScreenFont.y * (SCREEN_FONT_CX-1)) + writeCnt;
	u16 mCnt = (SCREEN_FONT_CX-1) * SCREEN_FONT_CY;

	return cnt >= mCnt ? TRUE : FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontSetNewLine()
{
	ST_SCREEN_FONT* p = &ScreenFont;

	if(p->x == 0)
	{
		//sς݂̏ꍇ
		return;
	}

	p->x  = 0;
	p->y += 1;

	//y`FbŃA̕\̑Oōs܂
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 ScreenFontGetX()
{
	return ScreenFont.x;
}
//---------------------------------------------------------------------------
EWRAM_CODE u16 ScreenFontGetY()
{
	return ScreenFont.y;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontSetXY(u16 x, u16 y)
{
	ScreenFont.x = x;
	ScreenFont.y = y;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontClearXY()
{
	ScreenFont.x = 0;
	ScreenFont.y = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenFontDrawRestore()
{
	ST_SCREEN_FONT* p = &ScreenFont;
	u16 x, y;

	for(y=0; y<SCREEN_FONT_CY; y++)
	{
		for(x=0; x<SCREEN_FONT_CX; x++)
		{
			FontLeafSetAttr(p->buf.attrNo[x][y]);
			ScreenFontDrawChr(x, y, p->buf.code[x][y]);
		}
	}
}





//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgInit()
{
	dummy = 0;
	ScreenImgClear();
	LibMode3SetScreenBuffer((u16*)&ScreenImg.buf);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgClear()
{
	CpuSet((u32)&dummy, (u32)ScreenImg.buf, ((240*160*2) / 4) | CPUSET_32BIT | CPUSET_SRC_FIX);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgUpdate()
{
	CpuSet((u32)ScreenImg.buf, (u32)VideoBuffer, ((240*160*2) / 4) | CPUSET_32BIT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgLoadBg(u8* pBg)
{
	CpuSet((u32)pBg, (u32)ScreenImg.buf, ((240*160*2) / 4) | CPUSET_32BIT);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgLoadChr(u8* pChr, u8 pos)
{
	ST_IMG_HEADER* h = (ST_IMG_HEADER*)pChr;
	u16 sx = 0;
	u16 sy = SCREEN_CY - h->cy;

	switch(pos)
	{
	case 0:		//
		sx = 10;
		break;

	case 1:		//E
		sx = SCREEN_CX - h->cx - 10;
		break;

	case 2:		//
		sx = (SCREEN_CX / 2) - (h->cx / 2);
		break;
	}

	//IWRAM̈ֈړ܂iiwram_arm.cj
	Mode3DrawImage2(sx, sy, h->cx, h->cy, (u16*)(h+1));
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgFlash()
{
	ScreenImgFadeIn(BLEND_MODE_LIGHT, 0, 15, 20);
	ScreenImgFadeOut(20);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgFadeIn(u16 mode, u16 min, u16 max, u16 wait)
{
	Mode3DrawBlend(mode, min, max);
	Mode3DrawFadeIn(wait);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenImgFadeOut(u16 wait)
{
	Mode3DrawFadeOut(wait);
}





//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCursorInit()
{
	ST_CURSOR* p = &Cursor;

	p->x         = 0;
	p->y         = SCREEN_CY;
	p->sprNo     = CURSOR_SPRITE_NO;
	p->attr[0]   = CURSOR_TILE_NO_PAGE;
	p->attr[1]   = CURSOR_TILE_NO_KEY;
	p->attrNo    = 0;
	p->tileNo    = p->attrNo;
	p->blinkFlag = 0x00;

	SpriteSetSize(p->sprNo, SP_SIZE_16, SP_SQUARE, SP_COLOR_16);
}
//---------------------------------------------------------------------------
//IRQUserHandler֐iiwram_arm.cjĂ΂܂
EWRAM_CODE void ScreenCursorDrawBlink()
{
	ST_CURSOR* p = &Cursor;

	if( _UMod(Timer.clockTick, CURSOR_BLINK_TIME) != 0 )
	{
		return;
	}

	if(p->blinkFlag == TRUE)
	{
		p->tileNo = p->attr[ p->attrNo ];
	}
	else
	{
		p->tileNo = CURSOR_TILE_NO_CLS;
	}
	p->blinkFlag ^= 0x01;

	SpriteSetChr(p->sprNo, p->tileNo);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCursorMoveIn()
{
	ST_SCREEN_FONT* p = &ScreenFont;
	ST_FONT_LEAF*   f = &FontLeaf;
	ST_CURSOR*      c = &Cursor;

	c->x = CURSOR_SX + p->x * (f->cx);
	c->y = CURSOR_SY + p->y * (f->cy + 2);		//+2͗]

	SpriteMove(c->sprNo, c->x, c->y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCursorMoveOut()
{
	SpriteMove(Cursor.sprNo, 0, SCREEN_CY);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCursorSetAttr(u16 no)
{
	Cursor.attrNo = no;
}
